package bin.ns;

import lib.CLIAppBase;

using tink.CoreApi;

@:build(macros.Binstore.includeBin("NameSystemCLI", ["ns-cli"]))
class NameSystemCLI extends CLIAppBase {
	public function new() {
		registerAsyncSubcommand("get", (args) -> {
			if (args.length < 1) {
				return Future.sync(false);
			}

			var ns = NameSystemRPC.getDefault();

			return ns.getIDRecord(args[0]).map((r) -> {
				switch r {
					case Success(data):
						handle.writeLine('Resolved: $data');
					case Failure(failure):
						handle.writeLine('Failed: $failure');
				}

				return true;
			});
		}, "<name>");

		registerAsyncSubcommand("register", (args) -> {
			if (args.length < 2) {
				return Future.sync(false);
			}

			var id = Std.parseInt(args[1]);

			if (id == null) {
				handle.writeLine("Not a valid id");
				return Future.sync(false);
			}

			var ns = NameSystemRPC.getDefault();

			return ns.setIDRecord(args[0], id).map((r) -> {
				switch r {
					case Success(_):
						handle.writeLine('Set');
					case Failure(failure):
						handle.writeLine('Failed: $failure');
				}

				return true;
			});
		}, "<name> <id>");

		registerAsyncSubcommand("list", (args) -> {
			var ns = NameSystemRPC.getDefault();

			return ns.listIDRecords().map((r) -> {
				switch r {
					case Success(data):
						handle.writeLine('List: $data');
					case Failure(failure):
						handle.writeLine('Failed: $failure');
				}

				return true;
			});
		});

		registerAsyncSubcommand("unregister", (args) -> {
			if (args.length < 1) {
				return Future.sync(false);
			}

			var ns = NameSystemRPC.getDefault();

			return ns.removeIDRecord(args[0]).map((r) -> {
				switch r {
					case Success(_):
						handle.writeLine('Unregisterd');
					case Failure(failure):
						handle.writeLine('Failed: $failure');
				}

				return true;
			});
		}, "<name>");
	}
}
